@@ -35,6 +35,19 @@ def clickhouse_default(expression: str) -> ClickhouseDefault:
     return ClickhouseDefault(expression=expression)
 
 
+@dataclasses.dataclass(frozen=True)
+class ClickHouseTTL:
+    expression: str
+
+
+@dataclasses.dataclass(frozen=True)
+class ClickHouseJson:
+    max_dynamic_paths: int | None = None
+    max_dynamic_types: int | None = None
+    skip_paths: tuple[str, ...] = ()
+    skip_regexps: tuple[str, ...] = ()
+
+
 def clickhouse_decimal(precision: int, scale: int) -> Type[Decimal]:
     return Annotated[Decimal, Field(max_digits=precision, decimal_places=scale)]
 
@@ -48,6 +61,14 @@ def clickhouse_datetime64(precision: int) -> Type[datetime]:
     return Annotated[datetime, ClickhousePrecision(precision=precision)]
 
 
+type Point = Annotated[tuple[float, float], "Point"]
+type Ring = Annotated[list[tuple[float, float]], "Ring"]
+type LineString = Annotated[list[tuple[float, float]], "LineString"]
+type MultiLineString = Annotated[list[list[tuple[float, float]]], "MultiLineString"]
+type Polygon = Annotated[list[list[tuple[float, float]]], "Polygon"]
+type MultiPolygon = Annotated[list[list[list[tuple[float, float]]]], "MultiPolygon"]
+
+
 def aggregated[T](
         result_type: Type[T],
         agg_func: str,
@@ -70,6 +91,46 @@ class AggregateFunction:
         }
 
 
+def simple_aggregated[T](
+        agg_func: str,
+        arg_type: Type[T]
+) -> Type[T]:
+    """Helper to create a SimpleAggregateFunction type annotation.
+
+    SimpleAggregateFunction is a ClickHouse type for storing aggregated values directly
+    instead of intermediate states. It's more efficient for functions like sum, max, min, etc.
+
+    Args:
+        agg_func: The aggregation function name (e.g., "sum", "max", "anyLast")
+        arg_type: The argument type for the function (also the result type)
+
+    Returns:
+        An Annotated type with SimpleAggregateFunction metadata
+
+    Example:
+        ```python
+        from moose_lib import simple_aggregated
+
+        row_count: simple_aggregated("sum", int)
+        max_value: simple_aggregated("max", float)
+        last_status: simple_aggregated("anyLast", str)
+        ```
+    """
+    return Annotated[arg_type, SimpleAggregateFunction(agg_func=agg_func, arg_type=arg_type)]
+
+
+@dataclasses.dataclass(frozen=True)
+class SimpleAggregateFunction:
+    agg_func: str
+    arg_type: type | GenericAlias | _BaseGenericAlias
+
+    def to_dict(self):
+        return {
+            "functionName": self.agg_func,
+            "argumentType": py_type_to_column_type(self.arg_type, [])[2]
+        }
+
+
 def enum_value_serializer(value: int | str):
     if isinstance(value, int):
         return {"Int": value}
@@ -107,7 +168,15 @@ class MapType(BaseModel):
     value_type: "DataType"
 
 
-type DataType = str | DataEnum | ArrayType | Nested | NamedTupleType | MapType
+class JsonOptions(BaseModel):
+    max_dynamic_paths: int | None = None
+    max_dynamic_types: int | None = None
+    typed_paths: list[tuple[str, "DataType"]] = []
+    skip_paths: list[str] = []
+    skip_regexps: list[str] = []
+
+
+type DataType = str | DataEnum | ArrayType | Nested | NamedTupleType | MapType | JsonOptions
 
 
 def handle_jwt(field_type: type) -> Tuple[bool, type]:
@@ -146,6 +215,7 @@ class Column(BaseModel):
     primary_key: bool
     default: str | None = None
     annotations: list[Tuple[str, Any]] = []
+    ttl: str | None = None
 
     def to_expr(self):
         # Lazy import to avoid circular dependency at import time
@@ -153,6 +223,53 @@ class Column(BaseModel):
         return ColumnRef(self)
 
 
+def _is_point_type(t: type) -> bool:
+    origin = get_origin(t)
+    if origin is tuple:
+        args = get_args(t)
+        return len(args) == 2 and all(arg is float for arg in args)
+    return False
+
+
+def _is_list_of(inner_check: Any, t: type) -> bool:
+    origin = get_origin(t)
+    if origin is list:
+        args = get_args(t)
+        return len(args) == 1 and inner_check(args[0])
+    return False
+
+
+def _validate_geometry_type(requested: str, t: type) -> None:
+    """
+    Validates that the provided Python type matches the expected structure
+    for the requested geometry annotation.
+    """
+    match requested:
+        case "Point":
+            if not _is_point_type(t):
+                raise ValueError("Point must be typed as tuple[float, float]")
+        case "Ring" | "LineString":
+            if not _is_list_of(_is_point_type, t):
+                raise ValueError(
+                    f"{requested} must be typed as list[tuple[float, float]]"
+                )
+        case "MultiLineString" | "Polygon":
+            if not _is_list_of(lambda x: _is_list_of(_is_point_type, x), t):
+                raise ValueError(
+                    f"{requested} must be typed as list[list[tuple[float, float]]]"
+                )
+        case "MultiPolygon":
+            if not _is_list_of(
+                    lambda x: _is_list_of(lambda y: _is_list_of(_is_point_type, y), x),
+                    t,
+            ):
+                raise ValueError(
+                    "MultiPolygon must be typed as list[list[list[tuple[float, float]]]]"
+                )
+        case _:
+            raise ValueError(f"Unknown geometry type annotation: {requested}")
+
+
 def py_type_to_column_type(t: type, mds: list[Any]) -> Tuple[bool, list[Any], DataType]:
     # handle Annotated[Optional[Annotated[...], ...]
     t, mds = handle_annotation(t, mds)
@@ -213,6 +330,23 @@ def py_type_to_column_type(t: type, mds: list[Any]) -> Tuple[bool, list[Any], Da
         data_type = "IPv4"
     elif t is ipaddress.IPv6Address:
         data_type = "IPv6"
+    elif any(md in [  # this check has to happen before t is matched against tuple/list
+        "Point",
+        "Ring",
+        "LineString",
+        "MultiLineString",
+        "Polygon",
+        "MultiPolygon",
+    ] for md in mds):
+        data_type = next(md for md in mds if md in [
+            "Point",
+            "Ring",
+            "LineString",
+            "MultiLineString",
+            "Polygon",
+            "MultiPolygon",
+        ])
+        _validate_geometry_type(data_type, t)
     elif get_origin(t) is list:
         inner_optional, _, inner_type = py_type_to_column_type(get_args(t)[0], [])
         data_type = ArrayType(element_type=inner_type, element_nullable=inner_optional)
@@ -229,6 +363,45 @@ def py_type_to_column_type(t: type, mds: list[Any]) -> Tuple[bool, list[Any], Da
         data_type = "UUID"
     elif t is Any:
         data_type = "Json"
+    elif any(isinstance(md, ClickHouseJson) for md in mds) and issubclass(t, BaseModel):
+        # Annotated[SomePydanticClass, ClickHouseJson(...)]
+        columns = _to_columns(t)
+        for c in columns:
+            if c.default is not None:
+                raise ValueError(
+                    "Default in inner field. Put ClickHouseDefault in top level field."
+                )
+        # Enforce extra='allow' for JSON-mapped models
+        if t.model_config.get('extra') != 'allow':
+            raise ValueError(
+                f"Model {t.__name__} with ClickHouseJson must have model_config with extra='allow'. "
+                "Add: model_config = ConfigDict(extra='allow')"
+            )
+        opts = next(md for md in mds if isinstance(md, ClickHouseJson))
+
+        # Build typed_paths from fields as tuples of (name, type)
+        typed_paths: list[tuple[str, DataType]] = []
+        for c in columns:
+            typed_paths.append((c.name, c.data_type))
+
+        has_any_option = (
+                opts.max_dynamic_paths is not None or
+                opts.max_dynamic_types is not None or
+                len(typed_paths) > 0 or
+                len(opts.skip_paths) > 0 or
+                len(opts.skip_regexps) > 0
+        )
+
+        if not has_any_option:
+            data_type = "Json"
+        else:
+            data_type = JsonOptions(
+                max_dynamic_paths=opts.max_dynamic_paths,
+                max_dynamic_types=opts.max_dynamic_types,
+                typed_paths=typed_paths,
+                skip_paths=list(opts.skip_paths),
+                skip_regexps=list(opts.skip_regexps),
+            )
     elif get_origin(t) is Literal and all(isinstance(arg, str) for arg in get_args(t)):
         data_type = "String"
         mds.append("LowCardinality")
@@ -280,6 +453,10 @@ def _to_columns(model: type[BaseModel]) -> list[Column]:
                 annotations.append(
                     ("aggregationFunction", md.to_dict())
                 )
+            if isinstance(md, SimpleAggregateFunction):
+                annotations.append(
+                    ("simpleAggregationFunction", md.to_dict())
+                )
             if md == "LowCardinality":
                 annotations.append(
                     ("LowCardinality", True)
@@ -293,6 +470,12 @@ def _to_columns(model: type[BaseModel]) -> list[Column]:
             None,
         )
 
+        # Extract TTL expression from metadata, if provided
+        ttl_expr = next(
+            (md.expression for md in mds if isinstance(md, ClickHouseTTL)),
+            None,
+        )
+
         columns.append(
             Column(
                 name=column_name,
@@ -302,6 +485,7 @@ def _to_columns(model: type[BaseModel]) -> list[Column]:
                 primary_key=primary_key,
                 default=default_expr,
                 annotations=annotations,
+                ttl=ttl_expr,
             )
         )
     return columns
